<?php if (!defined('PmWiki')) exit();
/*
 * ASCIIMath - Display MathML rendered ascii formula into PmWiki 2.x pages
 * Copyright 2005-2019 by D.Faure (dfaure@cpan.org)
 * ASCIIMathPHP written by and (c) Kee-Lin Steven Chan (kc56@cornell.edu)
 * ASCIIMath written by and (c) Peter Jipsen (http://www.chapman.edu/~jipsen)
 *
 * You can redistribute it and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * See http://www.pmwiki.org/wiki/Cookbook/ASCIIMath for info.
 */
$RecipeInfo['ASCIIMath']['Version'] = '20190209';

SDV($ASCIIMathOutputPubDir, 'asciimath');

SDV($HTMLStylesFmt['formula'], "
div.eqn { margin: 0 2em; }
div.eqn table { width: 100%; }
div.eqn iframe {
 width: 100%;
 height: 80px;
}");
/*
Markup('eqn', '>[=',
"/\\(:eqn(\\s+.*)?\\s*:\\)\\s+$KeepToken(\\d.*?)$KeepToken/e",
"Equation(\$pagename,PSS('$1'),\$GLOBALS['KPV']['$2'])");

Markup('eqnend', '>eqn',
"/\\(:eqn(\\s+.*)?\\s*:\\)\n(?s:(.*?))\\(:eqnend:\\)/e",
"Equation(\$pagename,PSS('$1'),PSS('$2'))");

Markup('{$', '>[=',
  '/\\{\\$\\s*(%([^%]+)%)?(?s:(.*?))\\$\\}/e',
  "Equation(\$pagename,PSS('$2'),PSS('$3'))");
*/

Markup('eqn', '>[=',
"/\\(:eqn(\\s+.*)?\\s*:\\)\\s+$KeepToken(\\d.*?)$KeepToken/",
"EquationMarkupA");

function EquationMarkupA($m) {
  extract($GLOBALS['MarkupToHTML']); # to get $pagename
  return Equation($pagename, $m[1], $GLOBALS['KPV'][$m[2]]);
}

Markup('eqnend', '>eqn',
"/\\(:eqn(\\s+.*)?\\s*:\\)\n(?s:(.*?))\\(:eqnend:\\)/",
"EquationMarkupB");

function EquationMarkupB($m) {
  extract($GLOBALS['MarkupToHTML']); # to get $pagename
  return Equation($pagename, $m[1], $m[2]);
}

Markup('{$', '>[=',
  '/\\{\\$\\s*(%([^%]+)%)?(?s:(.*?))\\$\\}/',
  "EquationMarkupC");

function EquationMarkupC($m) {
  extract($GLOBALS['MarkupToHTML']); # to get $pagename
  return Equation($pagename, $m[2], $m[3]);
}

SDV($GUIButtons['math'], array(1000, '{$ ', ' $}', '\\\\sqrt{n}', 
  '$FarmPubDirUrl/guiedit/math.gif"$[Math formula (ASCIIMathML)]"'));

function Equation($pagename, $args, $ascii) {
  global $ASCIIMathParams, $FarmD, $PubDirUrl, $ASCIIMathOutputPubDir, $action;
  $opt = array_merge((array)$ASCIIMathParams, ParseArgs($args));
  if($opt['height']) {
    $styles .= "height:$opt[height];";
    unset($opt['height']);
  }
  $renderNum = (@isset($opt['num']) or @in_array('num', $opt[''])) ? true : false;
  static $num = 0; $num = $opt['num'] ? $opt['num'] : $num + 1;
  unset($opt['num']);
  static $realNum = 0; $realNum++;
  $basename = "$pagename.$realNum";
  $name = $basename . '.' . md5($ascii) . ".xml";
  $url = "$PubDirUrl/$ASCIIMathOutputPubDir/$name";
  $dir = "$FarmD/pub/$ASCIIMathOutputPubDir";
  $filename = "$dir/$name";
  mkdirp($dir); fixperms($dir);
  if(!file_exists($filename) or $action == 'edit') {
    purgePreviousEquationIfNeeded($dir, $basename);
    # undo PmWiki's htmlspecialchars conversion
    $ascii = str_replace(
                  array('<:vspace>', '&lt;', '&gt;', '&amp;'),
                  array('', '<', '>', '&'),
                  $ascii);
    unset($opt['#'], $opt['-'], $opt['+'], $opt['']);
    global $ASCIIMathPath, $symbol_arr;
    SDV($ASCIIMathPath, dirname(__FILE__) . '/asciimath');
    include_once("$ASCIIMathPath/ASCIIMathPHP.cfg.php");
    require_once("$ASCIIMathPath/ASCIIMathPHP.class.php");

    global $MathMLNSAlias, $PMathML_XSL, $MathMLStartFmt, $MathMLEndFmt,
           $MathMLStyles;
    SDV($MathMLNSAlias, 'mml');
    $MathMLNS = ($MathMLNSAlias == 'm') ? "" :
      "xmlns:$MathMLNSAlias='http://www.w3.org/1998/Math/MathML'";
    SDV($PMathML_XSL, "pmathml.xsl");
    SDV($MathMLStartFmt,
"<?xml version='1.0'?>\n<?xml-stylesheet type='text/xsl' href='$PMathML_XSL'?>
<html xmlns='http://www.w3.org/1999/xhtml' $MathMLNS>
<head/><body>\n");
    SDV($MathMLEndFmt, "</body></html>");
    SDVA($MathMLStyles, array('displaystyle' => 'true'));

    $ascii_math = new ASCIIMathPHP($symbol_arr, $ascii, $MathMLNSAlias);
    $ascii_math->genMathML(array_merge($MathMLStyles, $opt));

    $mathml = array();
    $mathml[] = $MathMLStartFmt;
    $mathml[] = $ascii_math->getMathML();
    $mathml[] = $MathMLEndFmt;
    file_put_contents($filename, implode('', $mathml));
  }
  $output = array();
  $output[] = "<div class='eqn'><a name='eqn_$realNum'></a><table><tr>
<td><iframe id='eqn_$realNum' style='$styles' frameborder='0'
            src='$url'></iframe></td>";
  if($renderNum) $output[] = "<th>($num)</th>";
  $output[] = "</tr></table></div>";
  return '<:block>'.Keep(implode('', $output));
}

function purgePreviousEquationIfNeeded($dir, $basename, $once = true) {
  $dfp = opendir($dir); if ($dfp) return;
  while(($f = readdir($dfp)) !== false) {
    if($f{0} == '.') continue;
    if(strpos($f, $basename) !== false) {
      @unlink("$dir/$f");
      if($once) break;
    }
  }
  closedir($dfp);
}

if ($action == 'purgeqns') $HandleActions['purgeqns'] = 'HandlePurgeEquations';

function HandlePurgeEquations($pagename) {
  global $FarmD, $PubDirUrl, $ASCIIMathOutputPubDir;
  $dir = "$FarmD/pub/$ASCIIMathOutputPubDir";
  purgePreviousEquationIfNeeded($dir, $pagename, false);
  Redirect($pagename);
}

if(!function_exists('file_put_contents')) {
  function file_put_contents($filename, $data, $file_append = false) {
    $fp = fopen($filename, (!$file_append ? 'w+' : 'a+'));
    if(!$fp)
      trigger_error('file_put_contents cannot write in file.', E_USER_ERROR);
    else {
      fputs($fp, $data);
      fclose($fp);
    }
  }
}
